@@ -54,10 +54,6 @@
 #define KEY_ACTIVATE_DELAY  KEY_DIR "/idle_delay"
 #define KEY_LOCK_DELAY      KEY_DIR "/lock_delay"
 #define KEY_CYCLE_DELAY     KEY_DIR "/cycle_delay"
-#define KEY_DPMS_ENABLED    KEY_DIR "/dpms_enabled"
-#define KEY_DPMS_STANDBY    KEY_DIR "/dpms_standby"
-#define KEY_DPMS_SUSPEND    KEY_DIR "/dpms_suspend"
-#define KEY_DPMS_OFF        KEY_DIR "/dpms_off"
 #define KEY_THEMES          KEY_DIR "/themes"
 
 enum {
@@ -499,6 +495,42 @@ activate_delay_value_changed_cb (GtkRange *range,
         config_set_activate_delay ((gint32)value);
 }
 
+static int
+compare_theme_names (char *name_a,
+                     char *name_b,
+                     char *id_a,
+                     char *id_b)
+{
+
+        if (! id_a) {
+                return 1;
+        } else if (! id_b) {
+                return -1;
+        }
+
+        if (strcmp (id_a, "__blank-only") == 0) {
+                return -1;
+        } else if (strcmp (id_b, "__blank-only") == 0) {
+                return 1;
+        } else if (strcmp (id_a, "__random") == 0) {
+                return -1;
+        } else if (strcmp (id_b, "__random") == 0) {
+                return 1;
+        } else if (strcmp (id_a, "__separator") == 0) {
+                return -1;
+        } else if (strcmp (id_b, "__separator") == 0) {
+                return 1;
+        }
+
+        if (! name_a) {
+                return 1;
+        } else if (! name_b) {
+                return -1;
+        }
+
+        return strcmp (name_a, name_b);
+}
+
 static int
 compare_theme  (GtkTreeModel *model,
                 GtkTreeIter  *a,
@@ -516,30 +548,7 @@ compare_theme  (GtkTreeModel *model,
         gtk_tree_model_get (model, a, ID_COLUMN, &id_a, -1);
         gtk_tree_model_get (model, b, ID_COLUMN, &id_b, -1);
 
-        if (! id_a)
-                return 1;
-        else if (! id_b)
-                return -1;
-
-        if (strcmp (id_a, "__blank-only") == 0)
-                return -1;
-        else if (strcmp (id_b, "__blank-only") == 0)
-                return 1;
-        else if (strcmp (id_a, "__random") == 0)
-                return -1;
-        else if (strcmp (id_b, "__random") == 0)
-                return 1;
-        else if (strcmp (id_a, "__separator") == 0)
-                return -1;
-        else if (strcmp (id_b, "__separator") == 0)
-                return 1;
-
-        if (! name_a)
-                return 1;
-        else if (! name_b)
-                return -1;
-
-        result = strcmp (name_a, name_b);
+        result = compare_theme_names (name_a, name_b, id_a, id_b);
 
         g_free (name_a);
         g_free (name_b);
@@ -625,18 +634,24 @@ setup_treeview_selection (GtkWidget *tree)
 
         theme = config_get_theme (&is_writable);
 
-        if (! is_writable)
+        if (! is_writable) {
                 gtk_widget_set_sensitive (tree, FALSE);
+        }
 
         model = gtk_tree_view_get_model (GTK_TREE_VIEW (tree));
 
         if (theme && gtk_tree_model_get_iter_first (model, &iter)) {
-                char *id;
 
                 do {
+                        char *id;
+                        gboolean found;
+
                         gtk_tree_model_get (model, &iter,
                                             ID_COLUMN, &id, -1);
-                        if (id && strcmp (id, theme) == 0) {
+                        found = (id && strcmp (id, theme) == 0);
+                        g_free (id);
+
+                        if (found) {
                                 path = gtk_tree_model_get_path (model, &iter);
                                 break;
                         }
@@ -871,12 +886,9 @@ time_to_string_text (long time)
                         /* minutes */
                         string = g_strdup_printf (_("%s"), mins);
                 }
-        } else if (sec > 0) {
+        } else {
                 /* seconds */
                 string = g_strdup_printf (_("%s"), secs);
-        } else {
-                /* 0 seconds */
-                string = g_strdup (_("0 seconds"));
         }
 
         g_free (hours);
@@ -969,8 +981,9 @@ key_changed_cb (GConfClient *client,
 
         key = gconf_entry_get_key (entry);
 
-        if (! g_str_has_prefix (key, KEY_DIR))
+        if (! g_str_has_prefix (key, KEY_DIR)) {
                 return;
+        }
 
         value = gconf_entry_get_value (entry);
 
@@ -994,6 +1007,15 @@ key_changed_cb (GConfClient *client,
                 } else {
                         invalid_type_warning (key);
                 }
+        } else if (strcmp (key, KEY_THEMES) == 0) {
+                if (value->type == GCONF_VALUE_LIST) {
+                        GtkWidget *treeview;
+
+                        treeview = glade_xml_get_widget (xml, "savers_treeview");
+                        setup_treeview_selection (treeview);
+                } else {
+                        invalid_type_warning (key);
+                }
         } else if (strcmp (key, KEY_ACTIVATE_DELAY) == 0) {
 
                 if (value->type == GCONF_VALUE_INT) {
