@@ -61,6 +61,7 @@ struct GSManagerPrivate
         guint        active : 1;
         guint        lock_active : 1;
 
+        guint        fading : 1;
         guint        dialog_up : 1;
 
         guint        lock_timeout_id;
@@ -70,6 +71,8 @@ struct GSManagerPrivate
 
         GSList      *themes;
         GSSaverMode  saver_mode;
+        GSFade      *fade;
+        guint        unfade_idle_id;
 };
 
 enum {
@@ -279,6 +282,8 @@ gs_manager_set_lock_active (GSManager *manager,
 {
         g_return_if_fail (GS_IS_MANAGER (manager));
 
+        gs_debug ("Setting lock active: %d", lock_active);
+
         if (manager->priv->lock_active != lock_active) {
                 GSList *l;
 
@@ -335,8 +340,9 @@ gs_manager_set_user_switch_enabled (GSManager *manager,
 static gboolean
 activate_lock_timeout (GSManager *manager)
 {
-        if (manager->priv->lock_enabled)
+        if (manager->priv->lock_enabled) {
                 gs_manager_set_lock_active (manager, TRUE);
+        }
 
         manager->priv->lock_timeout_id = 0;
 
@@ -695,6 +701,8 @@ static void
 gs_manager_init (GSManager *manager)
 {
         manager->priv = GS_MANAGER_GET_PRIVATE (manager);
+
+        manager->priv->fade = gs_fade_new ();
 }
 
 static void
@@ -735,6 +743,8 @@ gs_manager_finalize (GObject *object)
         manager->priv->activate_time = 0;
         manager->priv->lock_enabled = FALSE;
 
+        g_object_unref (manager->priv->fade);
+
         G_OBJECT_CLASS (parent_class)->finalize (object);
 }
 
@@ -840,8 +850,13 @@ manager_maybe_start_job_for_window (GSManager *manager,
                 return;
         }
 
-        if (! manager->priv->throttle_enabled) {
-                gs_job_start (job);
+        if (! manager->priv->dialog_up) {
+                if (! manager->priv->throttle_enabled) {
+                        if (! gs_job_is_running (job)) {
+                                gs_debug ("Starting job for window");
+                                gs_job_start (job);
+                        }
+                }
         }
 }
 
@@ -885,6 +900,32 @@ window_grab_broken_cb (GSWindow           *window,
         }
 }
 
+static gboolean
+unfade_idle (GSManager *manager)
+{
+        gs_debug ("resetting fade");
+        gs_fade_reset (manager->priv->fade);
+        manager->priv->unfade_idle_id = 0;
+        return FALSE;
+}
+
+
+static void
+remove_unfade_idle (GSManager *manager)
+{
+        if (manager->priv->unfade_idle_id > 0) {
+                g_source_remove (manager->priv->unfade_idle_id);
+                manager->priv->unfade_idle_id = 0;
+        }
+}
+
+static void
+add_unfade_idle (GSManager *manager)
+{
+        remove_unfade_idle (manager);
+        manager->priv->unfade_idle_id = g_idle_add ((GSourceFunc)unfade_idle, manager);
+}
+
 static gboolean
 window_map_event_cb (GSWindow  *window,
                      GdkEvent  *event,
@@ -929,9 +970,6 @@ manager_show_window (GSManager *manager,
 
         manager->priv->activate_time = time (NULL);
 
-        /* Set lock state to off initially.  It may be enabled by activate_lock_timout() */
-        gs_manager_set_lock_active (manager, FALSE);
-
         if (manager->priv->lock_timeout >= 0) {
                 remove_lock_timer (manager);
                 add_lock_timer (manager, manager->priv->lock_timeout);
@@ -942,6 +980,8 @@ manager_show_window (GSManager *manager,
                 add_cycle_timer (manager, manager->priv->cycle_timeout);
         }
 
+        add_unfade_idle (manager);
+
         /* FIXME: only emit signal once */
         g_signal_emit (manager, signals [ACTIVATED], 0);
 }
@@ -1094,6 +1134,15 @@ remove_job (GSJob *job)
         g_object_unref (job);
 }
 
+static void
+fade_done_cb (GSFade    *fade,
+              GSManager *manager)
+{
+        gs_debug ("fade completed, showing windows");
+        show_windows (manager->priv->windows);
+        manager->priv->fading = FALSE;
+}
+
 static gboolean
 gs_manager_activate (GSManager *manager)
 {
@@ -1132,15 +1181,12 @@ gs_manager_activate (GSManager *manager)
         /* fade to black and show windows */
         do_fade = TRUE;
         if (do_fade) {
-                GSFade *fade;
-
-                fade = gs_fade_new ();
-                gs_fade_set_timeout (fade, FADE_TIMEOUT);
-                gs_fade_now (fade);
-                show_windows (manager->priv->windows);
-                g_usleep (500000);
-                gs_fade_reset (fade);
-                g_object_unref (fade);
+                manager->priv->fading = TRUE;
+                gs_debug ("fading out");
+                gs_fade_async (manager->priv->fade,
+                               FADE_TIMEOUT,
+                               (GSFadeDoneFunc)fade_done_cb,
+                               manager);
         } else {
                 show_windows (manager->priv->windows);
         }
@@ -1217,9 +1263,15 @@ gs_manager_request_unlock (GSManager *manager)
         g_return_if_fail (GS_IS_MANAGER (manager));
 
         if (! manager->priv->active) {
+                gs_debug ("Request unlock but manager is not active");
                 return;
         }
 
+        if (manager->priv->fading) {
+                gs_debug ("Request unlock so finishing fade");
+                gs_fade_finish (manager->priv->fade);
+        }
+
         /* Find the screen that contains the pointer */
         display = gdk_display_get_default ();
         gdk_display_get_pointer (display, &screen, NULL, NULL, NULL);
@@ -1227,7 +1279,9 @@ gs_manager_request_unlock (GSManager *manager)
 
         /* Find the gs-window that is on that screen */
         for (l = manager->priv->windows; l; l = l->next) {
-                int num = gdk_screen_get_number (GTK_WINDOW (l->data)->screen);
+                int num;
+
+                num = gdk_screen_get_number (GTK_WINDOW (l->data)->screen);
                 if (num == screen_num) {
                         gs_window_request_unlock (GS_WINDOW (l->data));
                         break;
