@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -217,6 +217,12 @@ gs_window_clear (GSWindow *window)
         GdkColormap *colormap;
         GtkStateType state;
 
+        g_return_if_fail (GS_IS_WINDOW (window));
+
+        if (! GTK_WIDGET_VISIBLE (GTK_WIDGET (window))) {
+                return;
+        }
+
         gs_debug ("Clearing window");
 
         state = (GtkStateType) 0;
@@ -558,8 +564,11 @@ get_env_vars (GtkWidget *widget)
                 "SESSION_MANAGER",
                 "XAUTHORITY",
                 "XAUTHLOCALHOSTNAME",
+                "KRB5CCNAME",
+                "KRBTKFILE",
                 "LANG",
-                "LANGUAGE"
+                "LANGUAGE",
+                "RUNNING_UNDER_GDM"
         };
 
         env = g_ptr_array_new ();
@@ -993,6 +1002,11 @@ gs_window_request_unlock (GSWindow *window)
 
         gs_debug ("Requesting unlock");
 
+        if (! GTK_WIDGET_VISIBLE (GTK_WIDGET (window))) {
+                gs_debug ("Request unlock but window is not visible!");
+                return;
+        }
+
         if (window->priv->watch_id > 0) {
                 return;
         }
@@ -1192,9 +1206,10 @@ static void
 queue_key_event (GSWindow    *window,
                  GdkEventKey *event)
 {
-        /* Eat the first return or space */
+        /* Eat the first return, enter, or space */
         if (window->priv->key_events == NULL
             && (event->keyval == GDK_Return
+                || event->keyval == GDK_KP_Enter
                 || event->keyval == GDK_space)) {
                 return;
         }
@@ -1203,6 +1218,23 @@ queue_key_event (GSWindow    *window,
                                                    gdk_event_copy ((GdkEvent *)event));
 }
 
+static gboolean
+maybe_request_unlock (GSWindow *window)
+{
+        gboolean ret;
+
+        ret = FALSE;
+
+        /* if we don't already have a socket then request an unlock */
+        if (! window->priv->socket
+            && (window->priv->request_unlock_idle_id == 0)) {
+                add_request_unlock_idle (window);
+                ret = TRUE;
+        }
+
+        return ret;
+}
+
 static gboolean
 gs_window_real_key_press_event (GtkWidget   *widget,
                                 GdkEventKey *event)
@@ -1213,9 +1245,7 @@ gs_window_real_key_press_event (GtkWidget   *widget,
 
         /* if we don't already have a socket then request an unlock */
         if (! GS_WINDOW (widget)->priv->socket) {
-                if (GS_WINDOW (widget)->priv->request_unlock_idle_id == 0) {
-                        add_request_unlock_idle (GS_WINDOW (widget));
-                }
+                maybe_request_unlock (GS_WINDOW (widget));
 
                 catch_events = TRUE;
         } else {
@@ -1259,11 +1289,8 @@ gs_window_real_motion_notify_event (GtkWidget      *widget,
                         ABS (window->priv->last_y - event->y));
 
         if (distance > min_distance) {
-                /* if we don't already have a socket then request an unlock */
-                if (! window->priv->socket
-                    && (window->priv->request_unlock_idle_id == 0)) {
-                        add_request_unlock_idle (window);
-                }
+                maybe_request_unlock (window);
+
                 window->priv->last_x = -1;
                 window->priv->last_y = -1;
         }
@@ -1271,6 +1298,30 @@ gs_window_real_motion_notify_event (GtkWidget      *widget,
         return FALSE;
 }
 
+static gboolean
+gs_window_real_button_press_event (GtkWidget      *widget,
+                                   GdkEventButton *event)
+{
+        GSWindow *window;
+
+        window = GS_WINDOW (widget);
+        maybe_request_unlock (window);
+
+        return FALSE;
+}
+
+static gboolean
+gs_window_real_scroll_event (GtkWidget      *widget,
+                             GdkEventScroll *event)
+{
+        GSWindow *window;
+
+        window = GS_WINDOW (widget);
+        maybe_request_unlock (window);
+
+        return FALSE;
+}
+
 static void
 gs_window_real_size_request (GtkWidget      *widget,
                              GtkRequisition *requisition)
@@ -1348,6 +1399,8 @@ gs_window_class_init (GSWindowClass *klass)
         widget_class->unrealize           = gs_window_real_unrealize;
         widget_class->key_press_event     = gs_window_real_key_press_event;
         widget_class->motion_notify_event = gs_window_real_motion_notify_event;
+        widget_class->button_press_event  = gs_window_real_button_press_event;
+        widget_class->scroll_event        = gs_window_real_scroll_event;
         widget_class->size_request        = gs_window_real_size_request;
         widget_class->grab_broken_event   = gs_window_real_grab_broken;
 
